import re

from pathlib import Path
from ieg_cloud_scan.aws.s3 import S3
from ieg_cloud_scan.aws.rds import RDS
from ieg_cloud_scan.aws.ebs import EBS
from ieg_cloud_scan.aws.ec2 import EC2
from ieg_cloud_scan.utils.logger import logger
from ieg_cloud_scan.aws.medialive import MediaLive
from ieg_cloud_scan.utils.csv_writer import CsvWriter


class AWSInfo:
    def get_ebs_info(regions):
        region_volume_id_names = {}
        for region in regions:
            try:
                logger.log(f"Getting EBS info for {region}")
                ebs_instance = EBS(region=region)
                untagged_ebs = ebs_instance.volume_without_tag("AMGID")
                logger.log(f"Untagged {region} {len(untagged_ebs.keys())}")
                wrongly_tagged_ebs = ebs_instance.volume_with_mutuallyexclusive_tagvalues(
                    {"kubernetes.io/created-for/pvc/namespace": "monitoring"},
                    {"AMGID": "AMGX0999"},
                )
                logger.log(f"Wrongly Untagged {region} {len(wrongly_tagged_ebs.keys())}")
                volumeids_names = {}
                volumeids_names = {"untagged_volumes": untagged_ebs} if untagged_ebs else {}
                volumeids_names.update(
                    {"wronglytagged_volumes": wrongly_tagged_ebs}
                    if wrongly_tagged_ebs
                    else {}
                )
                region_volume_id_names.update(
                    {region: volumeids_names} if volumeids_names else {}
                )
            except Exception as e:
                logger.log(f"{region} : {e}")
        return region_volume_id_names

    def get_s3_info():
        s3_client = S3()
        global_buckets_size = {}
        logger.log(f"Getting S3 info")
        buckets_sizes = s3_client.list_buckets_and_sizes(tag_key="AMGID")
        global_buckets_size["global"] = buckets_sizes
        logger.log(
            f"Tagged S3 buckets {len(global_buckets_size['tagged_buckets'].keys())}"
        )
        logger.log(
            f"Untagged S3 buckets {len(global_buckets_size['untagged_buckets'].keys())}"
        )
        return global_buckets_size

    def get_ec2_info(regions):
        region_untagged_instance_id_names = {}
        for region in regions:
            logger.log(f"Getting EC2 info for {region}")
            ec2_client = EC2(region=region)
            untagged_instances = ec2_client.get_instance_ids_without_tag("Stack")
            logger.log(f"Without Stack tag {region} {len(untagged_instances.keys())}")
            region_untagged_instance_id_names.update(
                {region: untagged_instances}
            ) if untagged_instances else {}
        return region_untagged_instance_id_names

    def get_medialive_info(regions):
        region_untagged_medialive_id_names = {}
        for region in regions:
            logger.log(f"Getting Medialive info for {region}")
            ml_client = MediaLive(region=region)
            untagged_medialives = ml_client.get_medialive_instances_without_tag(
                tag_key="AMGID", tag_value_pattern=r"^AMG"
            )
            region_untagged_medialive_id_names.update(
                {region: untagged_medialives}
            ) if untagged_medialives else {}

        return region_untagged_medialive_id_names

    def get_medialive_destinations(regions):
        region_medialive_destinations = {}
        for region in regions:
            logger.log(f"Getting Medialive info for {region}")
            ml_client = MediaLive(region=region)
            medialive_destinations = ml_client.get_output_info_of_channels()
            region_medialive_destinations.update(
                {region: medialive_destinations}
            ) if medialive_destinations else {}

        return region_medialive_destinations

    def get_rds_info(regions):
        region_untagged_rds_arn_id = {}
        for region in regions:
            logger.log(f"Getting RDS info for {region}")
            rds_client = RDS(region=region)
            untagged_rds = rds_client.get_untagged_rds_instances(tag_key="AMGID")
            region_untagged_rds_arn_id.update(
                {region: untagged_rds}
            ) if untagged_rds else {}

        return region_untagged_rds_arn_id
