import boto3


class EBS:
    def __init__(self, region):
        self.ec2_client = boto3.client("ec2", region_name=region)
        self.response = self.ec2_client.describe_volumes()
        self.ondemand_volumes = self.get_ondemand_volumes()

    def get_ondemand_volumes(self):
        ondemand_volumes = []
        for volume in self.response["Volumes"]:
            if "Attachments" in volume and volume["Attachments"] and "Tags" in volume:
                ondemand_volumes.append(volume)
        return ondemand_volumes

    def volume_without_tag(self, tag_name):
        # Filter volumes without the tag_name tag and attached to on-demand instances
        filtered_volumes = {}
        for volume in self.ondemand_volumes:
            tags = {tag["Key"]: tag["Value"] for tag in volume["Tags"]}
            if tag_name not in tags or not tags[tag_name]:
                filtered_volumes[volume["VolumeId"]] = self.get_volume_name(tags)
        return filtered_volumes

    def get_volume_name(self, tags):
        # Check if the volume has a name tag
        if "Name" in tags:
            volume_name = tags["Name"]
        else:
            volume_name = "Notfound"  # Default value if name tag is not present
        return volume_name

    def volume_with_mutuallyexclusive_tagvalues(
        self, tag_name_value_1, tag_name_value_2
    ):
        tag1, value1 = next(iter(tag_name_value_1.items()))
        tag2, value2 = next(iter(tag_name_value_2.items()))
        # Create a dictionary to store the filtered volumes
        filtered_volumes = {}
        # Filter volumes without the tag_name tag and attached to on-demand instances
        for volume in self.ondemand_volumes:
            tags = {tag["Key"]: tag["Value"] for tag in volume["Tags"]}
            if tag1 in tags and tag2 in tags:
                # a generalised function has been written to catch kubernetes.io/created-for/pvc/namespace: monitoring to not be tagged with AMGID:AMGX0999
                namespace = tags.get(tag1)
                amgid = tags.get(tag2)
                if namespace != value1 and amgid == value2:
                    filtered_volumes[volume["VolumeId"]] = self.get_volume_name(tags)
        return filtered_volumes
