import boto3
from botocore.exceptions import ClientError
from ieg_cloud_scan.utils.logger import logger


class ElasticCache:
    def __init__(self, region):
        self.elasticache_client = boto3.client("elasticache", region_name=region)
        self.response = self.elasticache_client.describe_cache_clusters()

    def get_untagged_elastic_cache_instances(self, tag_key):
        untagged_ecs = {}
        untagged_tagkey_instances = {}

        for cluster in self.response["CacheClusters"]:
            cluster_arn = cluster["ARN"]
            cluster_name = cluster["CacheClusterId"]

            # Check if the cluster has tags
            if "Tags" in cluster:
                tags = {tag["Key"]: tag["Value"] for tag in cluster["Tags"]}

                # Check if the tag key is missing
                if tag_key not in tags:
                    logger.log(f"{cluster_arn} not tagged")
                    untagged_tagkey_instances.update({cluster_arn: cluster_name})
            else:
                untagged_tagkey_instances.update({cluster_arn: cluster_name})
        untagged_ecs = (
            {"untagged_ecs": untagged_tagkey_instances}
            if untagged_tagkey_instances
            else {}
        )
        logger.log(f"Untagged EC {len(untagged_tagkey_instances.keys())}")

        return untagged_ecs
