import boto3


class EC2:
    def __init__(self, region) -> None:
        self.ec2_client = boto3.client("ec2", region_name=region)

    def get_instance_ids_without_tag(self, tag_name):
        response = self.ec2_client.describe_instances()
        instances_ids_names = {
            "stack_untagged_instances": {}
        }  # we may add more category of untagged key checks
        instances = {}
        for reservation in response["Reservations"]:
            for instance in reservation["Instances"]:
                instance_id = instance["InstanceId"]
                instance_name = ""

                # Check if the instance has a Name tag
                if "Tags" in instance:
                    tags = {tag["Key"]: tag["Value"] for tag in instance["Tags"]}
                    if "Name" in tags:
                        instance_name = tags["Name"]

                # Check if the instance is on-demand
                instance_lifecycle = instance.get("InstanceLifecycle", "")
                if instance_lifecycle != "spot":
                    # Check if the instance has the Stack tag
                    if "Tags" not in instance or tag_name not in tags:
                        instances[instance_id] = instance_name
        instances_ids_names = {"stack_untagged_instances": instances} if instances else {}

        return instances_ids_names
