import boto3
from botocore.exceptions import ClientError
from ieg_cloud_scan.utils.logger import logger


class S3:
    def __init__(self):
        self.s3_client = boto3.client("s3")
        self.response = self.s3_client.list_buckets()

    def list_buckets_and_sizes(self, tag_key):
        buckets_sizes = {"tagged_buckets": {}, "untagged_buckets": {}}
        buckets = self.response["Buckets"]
        for bucket in buckets:
            bucket_name = bucket["Name"]
            try:
                response = self.s3_client.get_bucket_tagging(Bucket=bucket_name)
                tags = response["TagSet"]
                tag_keys = [tag["Key"] for tag in tags]

                # Check if the specified tag key is present
                if tag_key in tag_keys:
                    # Get bucket size
                    size = self.get_bucket_size(bucket_name)
                    buckets_sizes["tagged_buckets"][bucket_name] = size
                else:
                    # If the specified tag key is not present
                    buckets_sizes["untagged_buckets"][bucket_name] = None

            except ClientError as e:
                if e.response["Error"]["Code"] == "NoSuchTagSet":
                    # If the bucket has no tags, consider it as untagged
                    # Get bucket size
                    size = self.get_bucket_size(bucket_name)
                    buckets_sizes["untagged_buckets"][bucket_name] = size
                else:
                    # Handle other exceptions if needed
                    logger.log_error(
                        f"Error retrieving tags for bucket {bucket_name}: {e}"
                    )

        return buckets_sizes

    def get_bucket_size(self, bucket_name):
        try:
            response = self.s3_client.list_objects_v2(Bucket=bucket_name)
            size_in_bytes = sum(obj["Size"] for obj in response["Contents"])
            size_in_gb = size_in_bytes / (1024**3)
        except KeyError:
            # If the bucket is empty, consider the size as 0
            size_in_gb = 0

        return size_in_gb
