"""Helper methods to get info about secrets"""

import json

import boto3
from botocore.exceptions import ClientError

from ieg_cloud_scan.utils.logger import logger


class SecretManager:
    """Helper methods to get info about secrets"""

    def __init__(self, region: str) -> None:
        self.region = region
        self.sm_client = boto3.client("secretsmanager", region_name=region)

    def get_secrets(self, secret_name: str, secret_dict: dict = {}):
        """Takes a dict of keys which are secretstrings and returns dict with secrets as values"""
        output_secret_dict = {}

        # Decrypts secret using the associated KMS key.
        response = {}
        try:
            response = self.sm_client.get_secret_value(SecretId=secret_name)
        except ClientError as exception:
            # For a list of exceptions thrown, see
            # https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
            logger.log_error(
                f"Error retrieving {secret_name} in {self.region} Exception:{exception}"
            )
        if 'SecretString' in response:
            output_secret_dict = json.loads(response['SecretString'])
            for key in output_secret_dict.keys():
                secret_dict[key] = output_secret_dict[key]
        return secret_dict
