import time
import json
import jwt
import requests

project = 'ieg-iota'
zone = 'us-east1-a'
scopes = "openid"
expires_in = 3600


json_filename = 'google-auth.json'
endpoints = [
    "http://platform-cprod-cp305.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-warme.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-meuw1.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-musw2.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-wbsec.cloudport.amagi.tv/kubecost",
    "http://platform-cprod-nbcna.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-nbcuk.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-migrn.cloudport.amagi.tv/kubecost",
    # "http://platform-cprod-mase2.cloudport.amagi.tv/kubecost",
]


# YESTERDAY - 
# metric = "model/allocation/view?aggregate=namespace&window=yesterday&shareIdle=false&idle=true&idleByNode=false&includeSharedCostBreakdown=true&shareTenancyCosts=true&shareNamespaces=&shareCost=NaN&shareSplit=weighted&chartType=costovertime&costUnit=cumulative&step="

# LAST WEEK - 
metric = "model/allocation/view?aggregate=namespace&window=lastweek&shareIdle=false&idle=true&idleByNode=false&includeSharedCostBreakdown=true&shareTenancyCosts=true&shareNamespaces=&shareCost=NaN&shareSplit=weighted&chartType=costovertime&costUnit=cumulative&step="

def load_json_credentials(filename):
    ''' Load the Google Service Account Credentials from Json file '''

    with open(filename, 'r') as f:
        data = f.read()

    return json.loads(data)

def load_private_key(json_cred):
    ''' Return the private key from the json credentials '''

    return json_cred['private_key']

def create_signed_jwt(pkey, pkey_id, email, scope):
    ''' Create a Signed JWT from a service account Json credentials file
    This Signed JWT will later be exchanged for an Access Token '''

    # Google Endpoint for creating OAuth 2.0 Access Tokens from Signed-JWT
    auth_url = "https://www.googleapis.com/oauth2/v4/token"

    issued = int(time.time())
    expires = issued + expires_in   # expires_in is in seconds

    # Note: this token expires and cannot be refreshed. The token must be recreated

    # JWT Headers
    additional_headers = {
            'kid': pkey_id,
            "alg": "RS256",
            "typ": "JWT"    # Google uses SHA256withRSA
    }

    # JWT Payload
    payload = {
        "iss": email,       # Issuer claim
        "sub": email,       # Issuer claim
        "aud": auth_url,    # Audience claim
        "iat": issued,      # Issued At claim
        "exp": expires,     # Expire time
        "scope": scope      # Permissions
    }

    # Encode the headers and payload and sign creating a Signed JWT (JWS)
    sig = jwt.encode(payload, pkey, algorithm="RS256", headers=additional_headers)

    return sig

def exchangeJwtForAccessToken(signed_jwt):
    '''
    This function takes a Signed JWT and exchanges it for a Google OAuth Access Token
    '''

    auth_url = "https://www.googleapis.com/oauth2/v4/token"

    params = {
        "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
        "assertion": signed_jwt
    }

    r = requests.post(auth_url, data=params)

    if r.ok:
        # print(r.text)
        return(r.json()['access_token'], '')

    return None, r.text

def getKubecostData(accessToken, endpoint):
    endpoint = f"{endpoint}/{metric}"
    cookies = {"token": accessToken}
    # headers = {"Accept": "application/json"}
    headers = {"Authorization": f"Bearer {accessToken}", "Accept": "application/json"}

    resp = requests.get(endpoint, headers=headers, cookies=cookies)

    if resp.ok:
        # print(r.text)
        return resp.json()['data']

    return None, resp.text

def parseKubecostData(data):
    cp_namespaces = ["playout", "live", "automation"]
    customers = {}
    non_cp_namespace_cost = 0

    print("==== Total Cost ====\n" , data["totals"]["totalCost"], "\n====================")
    # print("Total Cost - " , data["totals"]["totalCost"])
    for item in data["items"]["items"]:
        namespace, namespace_cost = item["name"], item["totalCost"]

        try:
            customer_name, namespace_type = namespace.rsplit("-", 1)
            if namespace_type in cp_namespaces:
                if customer_name in customers.keys():
                    customers[customer_name][namespace] = namespace_cost
                    # customers[customer_name]["detailed_cost"]["namespace"] = item
                    customers[customer_name]["total"] = customers[customer_name]["total"] + namespace_cost
                else:
                    customers[customer_name] = {
                        "customer_name": customer_name,
                        "total": namespace_cost,
                        namespace: namespace_cost,
                        # "detailed_cost": {
                            # namespace: item
                        # }
                    }
            else:
                raise
        except:
            print(f"{namespace} is not a customer namespace. Cost - {namespace_cost}")
            non_cp_namespace_cost = non_cp_namespace_cost + namespace_cost

    customer_cost_list = []
    for customer, detailed_cost in customers.items():
        customer_cost_list.append(detailed_cost)
    customer_cost_list = sorted(customer_cost_list, key=lambda x: x["total"], reverse=True)


    print("\n==== Non Cloudport Namespaces Cost ====")
    print(non_cp_namespace_cost)

    print("\n==== Customer Level Cost ====")
    for customer in customer_cost_list:
        print(customer)


if __name__ == '__main__':
    cred = load_json_credentials(json_filename)

    private_key = load_private_key(cred)

    # print("...FETCHING SIGNED JWT...")
    s_jwt = create_signed_jwt(
            private_key,
            cred['private_key_id'],
            cred['client_email'],
            scopes)

    # print("...FETCHING ACCESS TOKEN...")
    token, err = exchangeJwtForAccessToken(s_jwt)

    if token is None:
        print('Error:', err)
        exit(1)

    for endpoint in endpoints:
        cluster_name = endpoint.split("platform-")[-1].split(".")[0]
        print(f"\nCLUSTER - {cluster_name}\n")
        # print("...FETCHING KUBECOST DATA...")
        data = getKubecostData(accessToken = token, endpoint=endpoint)

        # print("...PARSING KUBECOST DATA...\n")
        parseKubecostData(data=data)
