"""Place holder for some handy utilities"""
from collections import defaultdict

from ieg_cloud_scan.utils.logger import logger


class CommonUtils:
    """Set of common functions that can be used without class initialisation"""

    @staticmethod
    def merge_dicts_recursive(d1, d2, path=None):
        """Merges two dicts even if they have duplicate keys"""
        if path is None:
            path = []
        for key in d2:
            if key in d1:
                if isinstance(d1[key], dict) and isinstance(d2[key], dict):
                    logger.log(f"Duplicate key:{key}")
                    CommonUtils.merge_dicts_recursive(d1[key], d2[key], path + [str(key)])
                else:
                    # If one of the values is not a dict, you can't continue merging.
                    # You need to decide how to handle this case.
                    logger.log(f"conflicting data at {path}{[str(key)]}")
            else:
                d1[key] = d2[key]
        return d1

    @staticmethod
    def count_nested_elements(data, is_root=True):
        """returns a dict of count of key of keys"""
        if isinstance(data, set):
            return len(data)
        elif isinstance(data, dict):
            count_dict = {}
            # Only add 'count' key if it's not the root dictionary
            if not is_root:
                count_dict['count'] = len(data)
            for key, value in data.items():
                # When calling recursively, it is not root anymore
                count_dict[key] = CommonUtils.count_nested_elements(value, is_root=False)
            return count_dict
        elif isinstance(data, list):
            return len(data)
        else:
            return {}

    @staticmethod
    def defaultdict_to_dict(d):
        """converts defaultdict to dict"""
        if isinstance(d, defaultdict):
            d = {k: CommonUtils.defaultdict_to_dict(v) for k, v in d.items()}
        return d
