"""Github module to do basic repo operations"""

import os
import shutil
import time

import jwt
import requests
from git.repo import Repo

from ieg_cloud_scan.utils.logger import logger


class Github:
    """Class to initialise Git hub auth and do Github operations"""

    def __init__(self, app_id, installation_id, private_key) -> None:
        # Generate a JWT
        payload = {
            "iat": int(time.time()),  # Issued at time
            "exp": int(time.time()) + (10 * 60),  # JWT expiration time (10 minute maximum)
            "iss": app_id,  # GitHub App ID
        }
        jwt_token = jwt.encode(payload, private_key, algorithm="RS256")
        headers = {
            "Authorization": f"Bearer {jwt_token}",
            "Accept": "application/vnd.github.v3+json",
        }
        response = requests.post(
            f"https://api.github.com/app/installations/{installation_id}/access_tokens",
            timeout=30,
            headers=headers,
        )
        response.raise_for_status()
        self.installation_token = response.json()["token"]

    def clone(self, clone_url, branch_name, directory_to_clone_to):
        """Clones a repo to a destination directory"""
        # Check if the directory exists and remove it
        if os.path.exists(directory_to_clone_to):
            logger.log(f"{directory_to_clone_to} already exists. Removing")
            shutil.rmtree(directory_to_clone_to)
        logger.log(f"Cloning {clone_url} branch:{branch_name} to {directory_to_clone_to}")
        Repo.clone_from(
            f"https://x-access-token:{self.installation_token}@github.com/{clone_url}",
            directory_to_clone_to,
            branch=branch_name,
            # env={
            #     "GIT_ASKPASS": "true",
            #     "GIT_USERNAME": "x-access-token",
            #     "GIT_PASSWORD": self.installation_token,
            # },
            depth=1,
        )
