import logging
import logging.handlers
import inspect


class Logger:
    def __init__(self, log_file):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.DEBUG)

        # Console Handler
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.INFO)
        console_formatter = logging.Formatter("%(levelname)s - %(name)s - %(message)s")
        console_handler.setFormatter(console_formatter)
        self.logger.addHandler(console_handler)

        # File Handler
        file_handler = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=1024000, backupCount=5
        )
        file_handler.setLevel(logging.DEBUG)
        file_formatter = logging.Formatter(
            "%(asctime)s - %(levelname)s - %(name)s - %(message)s"
        )
        file_handler.setFormatter(file_formatter)
        self.logger.addHandler(file_handler)

    def log(self, message):
        caller_frame = inspect.currentframe().f_back
        caller_class = caller_frame.f_globals["__name__"]
        caller_method = caller_frame.f_code.co_name
        caller_info = f"{caller_class}.{caller_method}"
        self.logger.info(f"{caller_info} - {message}")

    def log_error(self, message):
        caller_frame = inspect.currentframe().f_back
        caller_class = caller_frame.f_globals["__name__"]
        caller_method = caller_frame.f_code.co_name
        caller_info = f"{caller_class}.{caller_method}"
        self.logger.error(f"{caller_info} - {message}")


logger = Logger("./awsinfo.txt")
